@@ -4,6 +4,7 @@ from datetime import datetime
 import time
 import pandas as pd
 from glob import glob
+from apex import amp
 
 
 class AverageMeter(object):
@@ -42,7 +43,8 @@ class TorchFitter:
                  folder='models',
                  verbose=0,
                  validation_scheduler=True,
-                 step_scheduler=False):
+                 step_scheduler=False,
+                 early_stopping=0):
 
         if type(loss) == type:
             self.loss_function = loss()
@@ -78,6 +80,8 @@ class TorchFitter:
 
         self.scheduler = scheduler_class(self.optimizer, **scheduler_params)
 
+        self.early_stopping = early_stopping
+
         self.validation_scheduler = validation_scheduler  # do scheduler.step after validation stage loss
         self.step_scheduler = step_scheduler  # do scheduler.step after optimizer.step
         self.log(f'Fitter prepared. Device is {self.device}')
@@ -93,32 +97,37 @@ class TorchFitter:
             returns a pandas DataFrame object with training history
         """
         training_history = []
+        es_epochs = 0
         for e in range(self.n_epochs):
             history = {'epoch': e}  # training history log
 
-            # update log
+            # Update log
             if self.verbose > 0:
                 lr = self.optimizer.param_groups[0]['lr']
                 timestamp = datetime.utcnow().isoformat()
                 self.log(f'\n{timestamp}\nLR: {lr}')
 
-            # Run one train step
+            # Run one training epoch
             t = time.time()
             summary_loss = self.train_one_epoch(train_loader)
             history['train'] = summary_loss.avg  # training loss
 
+            # Print training result
             self.log(f'[RESULT]: Train. Epoch: {self.epoch}, summary_loss: {summary_loss.avg:.5f}, '
                      f'time: {(time.time() - t):.5f}')
             self.save(f'{self.base_dir}/last-checkpoint.bin')
 
-            # Run one validation step
+            # Run epoch validation
             t = time.time()
             summary_loss = self.validation(validation_loader)
             history['val'] = summary_loss.avg  # validation loss
             history['lr'] = self.optimizer.param_groups[0]['lr']
 
+            # Print validation results
             self.log(f'[RESULT]: Val. Epoch: {self.epoch}, summary_loss: {summary_loss.avg:.5f}, '
                      f'time: {(time.time() - t):.5f}')
+
+            # Check if result is improved, then save model
             if summary_loss.avg < self.best_summary_loss:
                 savepath = f'{self.base_dir}/best-checkpoint-{str(self.epoch).zfill(3)}epoch.bin'
                 print(f'Validation loss improved from {self.best_summary_loss} to '
@@ -128,12 +137,22 @@ class TorchFitter:
                 self.save(savepath)
                 for path in sorted(glob(f'{self.base_dir}/best-checkpoint-*epoch.bin'))[:-3]:
                     os.remove(path)
+                es_epochs = 0  # reset early stopping count
+            else:
+                es_epochs += 1
+
+            # Check if Early Stopping condition is met
+            if (self.early_stopping > 0) & (es_epochs > self.early_stopping):
+                self.log(f'Early Stopping: {self.early_stopping} epochs with no improvement')
+                training_history.append(history)
+                break
 
             if self.validation_scheduler:
                 self.scheduler.step(metrics=summary_loss.avg)
 
             training_history.append(history)
             self.epoch += 1
+
         return pd.DataFrame(training_history).set_index('epoch')
 
     def validation(self, val_loader):
@@ -223,7 +242,7 @@ class TorchFitter:
                 path: path of the file to be loaded
         """
         checkpoint = torch.load(path)
-        self.model.model.load_state_dict(checkpoint['model_state_dict'])
+        self.model.load_state_dict(checkpoint['model_state_dict'])
         self.optimizer.load_state_dict(checkpoint['optimizer_state_dict'])
         self.scheduler.load_state_dict(checkpoint['scheduler_state_dict'])
         self.best_summary_loss = checkpoint['best_summary_loss']
@@ -309,3 +328,108 @@ class TorchFitterBoxes(TorchFitter):
                 self.scheduler.step()
 
         return summary_loss
+
+
+class TorchFitterHP(TorchFitter):
+
+    def __init__(self,
+                 model,
+                 device,
+                 loss,
+                 n_epochs=1,
+                 optimizer=None,
+                 lr=0.0001,
+                 scheduler_class=None,
+                 scheduler_params=None,
+                 folder='models',
+                 verbose=0,
+                 validation_scheduler=True,
+                 step_scheduler=False,
+                 early_stopping=0,
+                 opt_level='O1'):
+        super(TorchFitterHP, self).__init__(model,
+                                            device,
+                                            loss,
+                                            n_epochs,
+                                            optimizer,
+                                            lr,
+                                            scheduler_class,
+                                            scheduler_params,
+                                            folder,
+                                            verbose,
+                                            validation_scheduler,
+                                            step_scheduler,
+                                            early_stopping)
+        self.model, self.optimizer = amp.initialize(self.model, self.optimizer, opt_level=opt_level)
+
+    def train_one_epoch(self, train_loader):
+        """ Run one epoch on the train dataset
+            inputs:
+                train_loader: DataLoader containing the training dataset
+            outputs:
+                summary_loss: AverageMeter object with this epochs's average loss
+        """
+        self.model.train()  # set train mode
+        summary_loss = AverageMeter()  # object to track the average loss
+        t = time.time()
+
+        # run epoch
+        for step, (images, labels) in enumerate(train_loader):
+            if self.verbose > 0:
+                if step % self.verbose == 0:
+                    print(
+                        f'Train Step {step}/{len(train_loader)}, ' +
+                        f'summary_loss: {summary_loss.avg:.5f}, ' +
+                        f'time: {(time.time() - t):.5f}', end='\r'
+                    )
+            # extract images and labels from the dataloader
+            batch_size = images.shape[0]
+            images = images.to(self.device).float()
+            labels = labels.to(self.device)
+
+            self.optimizer.zero_grad()
+
+            # Output and loss
+            output = self.model(images)
+            loss = self.loss_function(output, labels)
+
+            # backpropagation
+            with amp.scale_loss(loss, self.optimizer) as scaled_loss:
+                scaled_loss.backward()
+
+            summary_loss.update(loss.detach().item(), batch_size)
+
+            self.optimizer.step()
+
+            if self.step_scheduler:
+                self.scheduler.step()
+
+        return summary_loss
+
+    def save(self, path):
+        """ Save model and other metadata
+        input:
+            path: path of the file to be saved
+        """
+        self.model.eval()
+        torch.save({
+                'model_state_dict': self.model.state_dict(),
+                'optimizer_state_dict': self.optimizer.state_dict(),
+                'scheduler_state_dict': self.scheduler.state_dict(),
+                'best_summary_loss': self.best_summary_loss,
+                'epoch': self.epoch,
+                'amp': amp.state_dict()
+        }, path)
+
+    def load(self, path):
+        """ Load model and other metadata
+            input:
+                path: path of the file to be loaded
+        """
+        checkpoint = torch.load(path)
+        self.model.load_state_dict(checkpoint['model_state_dict'])
+        self.optimizer.load_state_dict(checkpoint['optimizer_state_dict'])
+        self.scheduler.load_state_dict(checkpoint['scheduler_state_dict'])
+        self.best_summary_loss = checkpoint['best_summary_loss']
+        self.epoch = checkpoint['epoch'] + 1
+        amp.load_state_dict(checkpoint['amp'])
